<?php
/*
 * Created For BZ9.com
 * Author: Dave
 * Copyright: BZ9.com
 * Licence: Commercial, See included licence file.
 */
if(!defined("APPLICATION_PATH")){ die("No DIRECT Access"); }


class settings
{
    Private $Reg;
    Private $config = array();

    public function __construct($Registry,$install=true){
        $this->Reg = $Registry;
        $this->Reg->log->log("class_load","Settings Class");

        $route_methods = array("setup");
        $this->Reg->route->set_route("settings", $route_methods);
    }

    public function ConfSet($index,$value){
        $this->config[$index] = $value;
    }

    public function Config($index = null){
        if($index){
            if(isset($this->config[$index])){
                return $this->config[$index];
            } else {
                return false;
            }

        }
        return $this->config;
    }

    public function init()
    {

        $this->include_conf();
        $this->site_test();
        $this->site_vars();
        $this->install();
        $this->update();

        return;
    }

    private function site_test()
    {
        if (!version_compare(PHP_VERSION, '5.2.0', '>=')) {
            $this->Reg->error->raise_error("fatal","A minimum of php version 5.2 is required to continue, Please contact your host to update your current version of php to either match or exceed version 5.2","admin");
        }

        if(!function_exists('curl_init'))
        {
            $this->Reg->error->raise_error("fatal","Your server is not configured to run curl. Please check with your host to get this function enabled.","admin");
        }
    }


    private function include_conf()
    {
        if(file_exists(APPLICATION_PATH."/config/config.php"))
        {
            include(APPLICATION_PATH."/config/config.php");

            $database_type = isset($database_type) ? $database_type : '';
            $database_host = isset($database_host) ? $database_host : '';
            $database_name = isset($database_name) ? $database_name : '';
            $database_username = isset($database_username) ? $database_username : '';
            $database_password = isset($database_password) ? $database_password : '';
            $admin_username = isset($admin_username) ? $admin_username : '';
            $admin_password = isset($admin_password) ? $admin_password : '';
            $site_url = isset($site_url) ? $site_url : '';
            $site_url = rtrim($site_url,"/");

            $this->ConfSet("database_type", $database_type);
            $this->ConfSet("database_host", $database_host);
            $this->ConfSet("database_name", $database_name);
            $this->ConfSet("database_username", $database_username);
            $this->ConfSet("database_password", $database_password);
            $this->ConfSet("admin_username", $admin_username);
            $this->ConfSet("admin_password", $admin_password);
            $this->ConfSet("site_url", $site_url);
            return;
        } else {
        $this->Reg->error->raise_error("fatal","Config.php was not found. Please check your installation","admin");
        }
    }
    private function site_vars()
    {
        $this->ConfSet("script_version", "1.3");

        $old_ver = array("1.2","1.0","0.8");
        $this->ConfSet("old_version", $old_ver);

        if($this->Config("site_url") == ""){
        $protocol =(isset($_SERVER["HTTPS"]) && $_SERVER["HTTPS"]=="on") ? "https://" : "http://";
        $currentPath = $_SERVER['PHP_SELF'];
        $pathInfo = pathinfo($currentPath);
        $pos = strpos($pathInfo['dirname'],"admin.php");
        if ($pos) {
            $parts =  explode("/",$pathInfo['dirname']);
            $pathInfo['dirname'] = "";
            foreach($parts as $item)
            {
              if($item != "admin.php"){ break; }
                  if($item != "")
                  {
                  $pathInfo['dirname'] .= "/".$item;
                  }
            }
        }
        $site_url =  $protocol . $_SERVER['HTTP_HOST'] . $pathInfo['dirname'];

        $this->ConfSet("site_url", $site_url);
        }
        return;
    }

    private function install()
    {
        if(!isset($this->config['database_type'])){
            $this->Reg->error->raise_error("fatal","Config.php was not loaded. Please check your installation","admin");
        }
        if($this->config['database_type'] == ""){
            $this->Reg->error->raise_error("fatal","Database type has not been set within config.php.","admin");
        }

        if(isset($_POST['setup'])){ $this->setup(); exit; }

        if($this->config['database_type'] == "file"){
            $this->install_file();
            return;
        }
        if($this->config['database_type'] == "mysql"){
            $this->install_mysql();
            return;
        }
        $this->Reg->error->raise_error("fatal","Database type has not been correctly setup within Config.php. Please check your installation","admin");
    }

    private function install_file()
    {
        $errors = array();
        $errors['item'] = array();

        if($this->Config('admin_username') == ""){ $this->Reg->error->raise_error("general","Admin Username is Empty","admin"); }
        if($this->Config('admin_password') == ""){ $this->Reg->error->raise_error("general","Admin Password is Empty","admin"); }

        if($this->Reg->error->is_errors("general")){
            $this->Reg->error->raise_error("general","Please correct the above within config.php and refresh this page.","admin");
            $this->Reg->tpl->showview("error");
        }

        if(!$this->Reg->security->validate($this->Config('admin_username'),"match",'/^[A-Za-z0-9!_]{5,30}+$/i'))
        {
            $this->Reg->error->raise_error("general","Username can only contain alphanumeric characters and be between 5 and 30 characters long.","admin");
        }
        if(!$this->Reg->security->validate($this->Config('admin_password'),"match",'/^[A-Za-z0-9!_]{5,30}+$/i'))
        {
            $this->Reg->error->raise_error("general","Password can only contain alphanumeric characters ! and _ and also be between 5 and 30 characters long.","admin");
        }

        if($this->Reg->error->is_errors("general")){
            $this->Reg->error->raise_error("general","Please correct the above within config.php and refresh this page.","admin");
            $this->Reg->tpl->showview("error");
        }


        if(!file_exists(APPLICATION_PATH."/data/url.txt"))
        {
            $this->Reg->error->raise_error("fatal","Cannot find data/url.txt please check your installation.","admin");
        } else {
            if(!is_writable(APPLICATION_PATH."/data/url.txt")){
                $this->Reg->error->raise_error("fatal","The file data/url.txt is not writable. Please chmod this file 0777 using your ftp client.","admin");
            }
        }

        if(!file_exists(APPLICATION_PATH."/data/cache.txt"))
        {
            $this->Reg->error->raise_error("fatal","Cannot find data/cache.txt please check your installation.","admin");
        } else {
            if(!is_writable(APPLICATION_PATH."/data/cache.txt")){
                $this->Reg->error->raise_error("fatal","The file data/cache.txt is not writable. Please chmod this file 0777 using your ftp client.","admin");
            }
        }


        return;
    }

    private function install_mysql(){

        require_once APPLICATION_PATH . "includes/Qsql.php";
        $db = new db($this->config("database_username"), $this->config("database_password"), $this->config("database_name"), $this->config("database_host"));
        if ( $db->is_error() )
        {
            $this->Reg->error->raise_error("fatal","There was a problem connecting to the database. Please ensure you have the correct settings within config.php.","admin");
        }
        if(!isset($_SESSION['install_complete']))
        {
        //$showtablequery = "SHOW TABLES FROM ".$this->config("database_name");
            $showtablequery = "SHOW TABLES";
        if(!$tables = $db->query($showtablequery))
        {
            //do install
            $sql = "CREATE TABLE IF NOT EXISTS admin (
  id int(10) NOT NULL AUTO_INCREMENT,
  username varchar(50) NOT NULL,
  email varchar(255) NOT NULL,
  password varchar(50) NOT NULL,
  ip varchar(50) NOT NULL,
  lastlogin datetime NOT NULL,
  lastip varchar(50) NOT NULL,
  curlogin datetime NOT NULL,
  pkey varchar(40) NOT NULL,
  master enum('y','n'),
  PRIMARY KEY (id)
) ENGINE=InnoDB AUTO_INCREMENT=1 ;";

            $db->query($sql);
            if ( $db->is_error() )
            {
                $this->Reg->error->raise_error("fatal","There was a problem creating the admin database table. Please ensure you have the correct settings within config.php.","admin");
            }

            $sql = "CREATE TABLE IF NOT EXISTS shortlinks (
  id int(10) NOT NULL AUTO_INCREMENT,
  url varchar(255) NOT NULL,
  linktxt varchar(100) NOT NULL,
  cloaked enum('Yes','No'),
  name varchar(200) NOT NULL,
  descr text,
  status enum('Active','Inactive'),
  hits int(10) DEFAULT 0,
  sdefault varchar(3) NOT NULL,
  r_group varchar(100) NOT NULL,
  expire enum('Yes','No') NOT NULL DEFAULT 'No',
  expdate date NOT NULL,
  r_301 enum('Yes','No') NOT NULL DEFAULT 'No',
  stealth varchar(255) NOT NULL,
  PRIMARY KEY (id)
) ENGINE=InnoDB AUTO_INCREMENT=1 ;";

            $db->flush();
            $db->query($sql);
            if ( $db->is_error() )
            {
                $this->Reg->error->raise_error("fatal","There was a problem creating the shortlinks database table. Please ensure you have the correct settings within config.php.","admin");
            }

            $sql = "CREATE TABLE IF NOT EXISTS rdata (
  id int(10) NOT NULL AUTO_INCREMENT,
  rd_id varchar(255) NOT NULL,
  rd_data text,
  PRIMARY KEY (id)
) ENGINE=InnoDB AUTO_INCREMENT=1 ;";

            $db->flush();
            $db->query($sql);
            if ( $db->is_error() )
            {
                $this->Reg->error->raise_error("fatal","There was a problem creating the data database table. Please ensure you have the correct settings within config.php.","admin");
            }

            $sql = "INSERT INTO shortlinks values(null,'http://bz9.com','demolink','No','Demo Link','An initial link to get you going','Active',0,'Yes','','No','2013/01/01','No','')";

            $db->flush();
            $db->query($sql);
            if ( $db->is_error() )
            {
                $this->Reg->error->raise_error("fatal","There was a problem creating the initial link entry. Please ensure you have the correct settings within config.php.","admin");
            }

            $sql = "INSERT INTO rdata (id,rd_id,rd_data) VALUES (null,'update','1.2')";
            $db->flush();
            $db->query($sql);
            if ( $db->is_error() )
            {
                $this->Reg->error->raise_error("fatal","There was a problem updating the databse update information..","admin");
            }
            $sql = "INSERT INTO rdata (id,rd_id,rd_data) VALUES (null,'update','1.3')";
            $db->flush();
            $db->query($sql);
            if ( $db->is_error() )
            {
                $this->Reg->error->raise_error("fatal","There was a problem updating the databse update information..","admin");
            }

            $this->Reg->db = $db;
            $this->Reg->tpl->set_section("public");
            $this->Reg->tpl->showview("setup");
        }
        }
        //did setup complete
        $varadmin = $db->get_var("SELECT count(*) FROM admin");
        if($varadmin == 0){
            $this->Reg->tpl->set_section("public");
            $this->Reg->tpl->showview("setup");
        }

        //tables found
        $_SESSION['install_complete'] = true;
        $this->Reg->db = $db;
        return;

    }


    public function setup()
    {
        if(!isset($_POST['setup'])){ $this->Reg->tpl->show404(); }
        //temp db setup
        require_once APPLICATION_PATH . "includes/Qsql.php";
        $db = new db($this->config("database_username"), $this->config("database_password"), $this->config("database_name"), $this->config("database_host"));
        if ( $db->is_error() )
        {
            $this->Reg->error->raise_error("fatal","There was a problem connecting to the database. Please ensure you have the correct settings within config.php.","admin");
        }
        //check for init run
        //$showtablequery = "SHOW TABLES FROM ".$this->config("database_name");
        $showtablequery = "SHOW TABLES";
        if($tables = $db->query("$showtablequery"))
        {
            //has this run before
            $varadmin = $db->get_var("SELECT count(*) FROM admin");
            if($varadmin != 0){
                $this->Reg->tpl->set_section("public");
                $this->Reg->tpl->showview("login");
            }


        $data = array();
        $error = array();

        $data['admin_username'] = $this->Reg->security->get("admin_username","post");
        $data['admin_email'] = $this->Reg->security->get("admin_email","post","email");
        $data['admin_password'] = $this->Reg->security->get("admin_password","post");
        $data['admin_password2'] = $this->Reg->security->get("admin_password2","post");

        if($data['admin_username'] == ""){ array_push($error,"Admin Username is empty."); }
        if($data['admin_email'] == ""){ array_push($error,"Admin Email is empty."); }
        if($data['admin_password'] == ""){ array_push($error,"Admin Password is empty."); }
        if($data['admin_password'] != $data['admin_password2']){ array_push($error,"Admin Passwords do not match."); }

        if(count($error) > 0){
            $data['error'] = $error;
            $this->Reg->tpl->set_section("public");
            $this->Reg->tpl->showview("setup",$data);
        }

        if(!$this->Reg->security->validate($data['admin_username'],"match",'/^[a-zA-Z0-9!_]+$/i'))
        {
            array_push($error,"Username can only contain alphanumeric characters.");
        }

        if(!$this->Reg->security->validate($data['admin_email'],"email"))
        {
            array_push($error,"Email is not valid.");
        }

        if(!$this->Reg->security->validate($data['admin_password'],"match",'/^[a-zA-Z0-9!_]+$/i'))
        {
            array_push($error,"Password can only contain letters,numbers, ! and underscore( _ ) characters.");
        }

        if(count($error) > 0){
            $data['error'] = $error;
            $this->Reg->tpl->set_section("public");
            $this->Reg->tpl->showview("setup",$data);
        }

        $data['admin_password'] = md5($data['admin_password']);
        $data['admin_username'] = mysql_real_escape_string($data['admin_username']);
        $data['admin_email'] = mysql_real_escape_string($data['admin_email']);
        $data['admin_password'] = mysql_real_escape_string($data['admin_password']);


        $sql = "INSERT INTO admin (id,username,email,password,ip,lastlogin,lastip,curlogin,pkey,master)
VALUES (null,'".$data['admin_username']."', '".$data['admin_email']."','".$data['admin_password']."','','','','','','y')";

        $db->query($sql);
        if ( $db->is_error() )
        {
            $this->Reg->error->raise_error("fatal","There was a problem creating the admin account.","admin");
        }
            $_SESSION['install_complete'] = true;
            $this->Reg->error->raise_error("form","Setup Complete.","admin");
        $this->Reg->tpl->set_section("public");
        $this->Reg->tpl->showview("login",$data);
        }
        //no tables found
        $this->Reg->tpl->show404();
    }

    private function update()
    {
        if(!isset($_SESSION['update_done'])){
            if($this->config['database_type'] == "mysql")
            {
                $varupdate = $this->Reg->db->get_var("SELECT count(*) FROM rdata WHERE rd_id = 'update' AND rd_data = '".$this->config['script_version']."'");
                if($varupdate < 1)
                {
                    $this->do_update();
                }
            }
        }

        $_SESSION['update_done'] = true;
        return;
    }

    private function do_update()
    {
        if($this->config['database_type'] == "mysql")
        {
            $varupdate = $this->Reg->db->get_var("SELECT count(*) FROM rdata WHERE rd_id = 'update' AND rd_data = '1.2'");
            if($varupdate < 1)
            {
                $sql = "ALTER TABLE shortlinks ADD r_group VARCHAR(100)";
                $this->Reg->db->query($sql);
                $sql = "INSERT INTO rdata (id,rd_id,rd_data) VALUES (null,'update','1.2')";
                $this->Reg->db->query($sql);
            }

            if($this->config['script_version'] == '1.3')
            {

                $sql = "ALTER TABLE shortlinks ADD expire enum('Yes','No') NOT NULL DEFAULT 'No',
  ADD expdate date NOT NULL,
  ADD r_301 enum('Yes','No') NOT NULL DEFAULT 'No',
  ADD stealth varchar(255) NOT NULL";
                $this->Reg->db->query($sql);
                $sql = "UPDATE shortlinks SET expdate = '2014/01/01' WHERE expdate = '0000/00/00'";
                $this->Reg->db->query($sql);
                $sql = "INSERT INTO rdata (id,rd_id,rd_data) VALUES (null,'update','1.3')";
                $this->Reg->db->query($sql);

            }
        }

        return;
    }

}
